/*
 * Copyright (c) 2007-2010 Sonatype, Inc. All rights reserved.
 *
 * "Sonatype" is a trademark of Sonatype, Inc.
 */

package org.sonatype.tycho.p2.publisher.cli;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.maven.execution.MavenExecutionResult;
import org.codehaus.plexus.PlexusContainer;
import org.codehaus.plexus.component.repository.exception.ComponentLookupException;
import org.codehaus.plexus.logging.Logger;
import org.codehaus.plexus.logging.LoggerManager;
import org.codehaus.plexus.tools.cli.AbstractCli;
import org.codehaus.plexus.util.IOUtil;

/**
 * JAVADOC
 * 
 * @author Alin Dreghiciu
 */
public class AssemblerCli
    extends AbstractCli
{

    private static final String URL_OPTION = "d";

    private static final String DIR_OPTION = "t";

    private static final String LOCAL_REPO_OPTION = "l";

    private static final String REMOTE_REPO_OPTION = "r";

    private static final String OFFLINE_OPTION = "o";

    private static final String UPDATE_SNAPSHOTS = "U";

    private Options options;

    private boolean executionFailed;

    public static void main( final String[] args )
        throws Exception
    {
        final AssemblerCli cli = new AssemblerCli();

        final int status = cli.execute( args );

        if ( cli.executionFailed )
        {
            System.exit( 1 );
        }
        else
        {
            System.exit( status );
        }
    }

    @Override
    public Options buildCliOptions( final Options options )
    {
        this.options = options;

        options.addOption( OptionBuilder.withLongOpt( "descriptor-url" ).hasArg().withDescription( "Assembly descriptor URL" ).create( URL_OPTION ) );
        options.addOption( OptionBuilder.withLongOpt( "target-dir" ).hasArg().withDescription( "Target directory" ).create( DIR_OPTION ) );
        options.addOption( OptionBuilder.withLongOpt( "local-repository" ).hasArg().withDescription( "Local repository" ).create( LOCAL_REPO_OPTION ) );
        options.addOption( OptionBuilder.withLongOpt( "remote-repository" ).hasArg().withDescription( "Remote repository" ).create( REMOTE_REPO_OPTION ) );
        options.addOption( OptionBuilder.withLongOpt( "offline" ).withDescription( "Work offline" ).create( OFFLINE_OPTION ) );
        options.addOption( OptionBuilder.withLongOpt( "update-snapshots" ).withDescription( "Forces a check for updated releases and snapshots on remote repositories" ).create( UPDATE_SNAPSHOTS ) );

        return options;
    }

    @Override
    public void invokePlexusComponent( final CommandLine cli, final PlexusContainer plexus )
        throws Exception
    {
        final String urlOption = cli.getOptionValue( URL_OPTION );
        if ( urlOption == null || urlOption.trim().length() == 0 || cli.hasOption( HELP ) )
        {
            displayHelp();
        }
        else
        {
            if ( cli.hasOption( QUIET ) )
            {
                setLogLevel( plexus, Logger.LEVEL_DISABLED );
            }
            else if ( cli.hasOption( DEBUG ) )
            {
                setLogLevel( plexus, Logger.LEVEL_DEBUG );
            }
            else if ( cli.hasOption( ERRORS ) )
            {
                setLogLevel( plexus, Logger.LEVEL_ERROR );
            }

            final String dirOption = cli.getOptionValue( DIR_OPTION, "." );

            final File assemblyDir = new File( dirOption );
            if ( assemblyDir.exists() )
            {
                if ( assemblyDir.isFile() )
                {
                    throw new Exception( String.format( "Provided directory %s is a file not a directory", assemblyDir.getAbsolutePath() ) );
                }
            }

            if ( !cli.hasOption( QUIET ) )
            {
                System.err.printf( "Assembly descriptor: %s\n", urlOption );
                System.err.printf( "Target directory:    %s\n", assemblyDir.getAbsolutePath() );
            }

            InputStream inputStream;
            try
            {
                inputStream = new URL( urlOption ).openStream();
            }
            catch ( final MalformedURLException e )
            {
                final File file = new File( urlOption );
                if ( !file.exists() )
                {
                    throw new Exception( String.format( "Provided descriptor must be a valid URL or a path to an existing file" ) );
                }
                inputStream = new FileInputStream( file );
            }

            try
            {
                final MavenExecutionResult result =
                    plexus.lookup( MavenEmbedder.class ).setLocalRepository( cli.getOptionValue( LOCAL_REPO_OPTION ) ).setRemoteRepositories( cli.getOptionValues( REMOTE_REPO_OPTION ) ).setOffline( cli.hasOption( OFFLINE_OPTION ) ).setForceUpdate( cli.hasOption( UPDATE_SNAPSHOTS ) ).execute( inputStream, assemblyDir );

                if ( result.hasExceptions() )
                {
                    // TODO make a multi exception instead of first one
                    throw new Exception( result.getExceptions().get( 0 ) );
                }
            }
            finally
            {
                IOUtil.close( inputStream );
            }
        }
    }

    @Override
    public void displayHelp()
    {
        System.out.println();

        new HelpFormatter().printHelp( "assembler [options]", "\nOptions:", options, "\n" );
    }

    @Override
    protected void showError( final String message, final Exception e, final boolean show )
    {
        executionFailed = true;
        super.showError( message, e, show );
    }

    @Override
    protected int showFatalError( final String message, final Exception e, final boolean show )
    {
        executionFailed = true;
        return super.showFatalError( message, e, show );
    }

    @Override
    public CommandLine parse( final String[] args )
        throws ParseException
    {
        try
        {
            return super.parse( args );
        }
        catch ( final ParseException e )
        {
            executionFailed = true;
            throw e;
        }
    }

    @Override
    public String getPomPropertiesPath()
    {
        return "META-INF/maven/org.sonatype.runtime/p2-publisher-cli/pom.properties";
    }

    private void setLogLevel( final PlexusContainer plexus, final int logLevel )
        throws ComponentLookupException
    {
        plexus.lookup( LoggerManager.class ).setThresholds( logLevel );
    }

}
